package com.diagbot.model.ai;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.diagbot.client.CRFServiceClient;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.model.ai.process.*;
import com.diagbot.model.entity.Diag;
import com.diagbot.model.entity.GeneralDesc;
import com.diagbot.model.entity.Lis;
import com.diagbot.model.entity.Medicine;
import com.diagbot.model.entity.Usual;
import com.diagbot.model.entity.Vital;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.DiagLabel;
import com.diagbot.model.label.FamilyLabel;
import com.diagbot.model.label.MaritalLabel;
import com.diagbot.model.label.MenstrualLabel;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PersonalLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.model.label.VitalLabel;
import com.diagbot.util.Content;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.SearchData;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName : InHospitalDoc
 * @Description :
 * @Author : 楼辉荣
 * @Date: 2020-03-03 18:58
 */
@Component
public class BeHospitalizedAI extends ModelAI {
    /**
     * FirstCourseRecord_cx[病历首程]
     * PastFamily_cx[既往史家族史]
     * PersonalHistory_cx[个人史(月经史、婚育史)]
     * HPIForCX_cx[主诉、现病史、专科检查]
     * GeneralVital_cx[一般体格检查]
     * chief_present[邵逸夫医院主诉]
     * Diagnoses_cx[诊断]
     * Present_cx[现病史]
     */
    public static List<String> medicalTextType = Arrays.asList("FirstCourseRecord_cx", "PastFamily_cx", "PersonalHistory_cx", "HPIForCX_cx",
            "GeneralVital_cx", "chief_present", "Diagnoses_cx", "Present_cx");
    public static String entityRelationObject = "entity_relation_object";
    public static String outputs = "outputs";


    /**
     *
     * 类型       模型
     * 主诉	      主诉-现病史模型
     * 现病史	  主诉-现病史模型
     * 既往史	  既往家族史模型
     * 家族史	  既往家族史模型
     * 个人史	  个人史模型(吸烟、饮酒，冶游)
     * 婚育史	  个人史模型
     * 月经史	  个人史模型
     * 体征	      体征模型
     * 诊断	      诊断模型
     *
     * 化验       化验模型     已结构化，未使用
     * 辅检	      辅检模型     已结构化，未使用
     *
     *
     * @param inputInfo
     * @param wordCrfDTO
     * @param crfServiceClient
     */
    public void medrec(SearchData inputInfo, WordCrfDTO wordCrfDTO, CRFServiceClient crfServiceClient) {
        JSONArray crfContent = new JSONArray();
        if (inputInfo != null) {
            /* 主诉 */
            if (StringUtils.isNotBlank(inputInfo.getChief())) {
                String chiefText = inputInfo.getChief();
                if (StringUtil.isNotBlank(chiefText)) {
                    chiefText = "主诉：" + StringUtil.removeBlank(chiefText);//主诉入参 文本前需加 "主诉："
                    ChiefLabel chiefLabel = new ChiefLabel();
                    chiefLabel.setText(chiefText);
                    chiefLabel.setAiText(chiefText);
                    wordCrfDTO.setChiefLabel(chiefLabel);
                    putContent(crfContent, medicalTextType.get(7), chiefText, Content.chief);/* 2020-06-08修改主诉模型为现病史模型 */
                }
            }
            /* 现病史 */
            if (StringUtils.isNotBlank(inputInfo.getSymptom())) {
                String presentText = inputInfo.getSymptom();
                PresentLabel presentLabel = new PresentLabel();
                presentLabel.setText(presentText);
                presentLabel.setAiText(presentText);
                wordCrfDTO.setPresentLabel(presentLabel);
                putContent(crfContent, medicalTextType.get(5), presentText, Content.present);
            }
            /* 既往史 */
            if (StringUtils.isNotBlank(inputInfo.getPasts())) {
                String pastText = inputInfo.getPasts();
                PastLabel pastLabel = new PastLabel();
                pastLabel.setText(pastText);
                pastLabel.setAiText(pastText);
                wordCrfDTO.setPastLabel(pastLabel);
                putContent(crfContent, medicalTextType.get(1), pastText, Content.past);
            }
            /* 月经史文本需从个人史截取一部分文本拼接 */
            String concatMenstrual = "";
            /* 个人史 */
            if (StringUtils.isNotBlank(inputInfo.getPersonal())) {
                String personalText = inputInfo.getPersonal();
                PersonalLabel personalLabel = new PersonalLabel();
                personalLabel.setText(personalText);
                personalLabel.setAiText(personalText);
                wordCrfDTO.setPersonalLabel(personalLabel);
                putContent(crfContent, medicalTextType.get(2), personalText, Content.personal);
                concatMenstrual = getConcatMenstrual(concatMenstrual, personalText);
            }
            /* 婚育史 */
            if (StringUtils.isNotBlank(inputInfo.getMarital())) {
                String maritalText = inputInfo.getMarital();
                MaritalLabel maritalLabel = new MaritalLabel();
                maritalLabel.setText(maritalText);
                maritalLabel.setAiText(maritalText);
                wordCrfDTO.setMaritalLabel(maritalLabel);
                putContent(crfContent, medicalTextType.get(2), maritalText, Content.marriage);
            }
            /* 月经史 */
            if (StringUtils.isNotBlank(inputInfo.getMenstrual())) {
                String menstrualText = inputInfo.getMenstrual();
                MenstrualLabel menstrualLabel = new MenstrualLabel();
                menstrualLabel.setText(menstrualText);
                menstrualLabel.setAiText(menstrualText);
                wordCrfDTO.setMenstrualLabel(menstrualLabel);
                putContent(crfContent, medicalTextType.get(2), concatMenstrual, menstrualText, Content.menses);
            }
            /* 家族史 */
            if (StringUtils.isNotBlank(inputInfo.getFamily())) {
                String familyText = inputInfo.getFamily();
                FamilyLabel familyLabel = new FamilyLabel();
                familyLabel.setText(familyText);
                familyLabel.setAiText(familyText);
                wordCrfDTO.setFamilyLabel(familyLabel);
                putContent(crfContent, medicalTextType.get(1), familyText, Content.family);
            }
            /* 辅检 */
            if(ListUtil.isNotEmpty(inputInfo.getPacs())){
                PacsLabel pacsLabel = new PacsLabel();
                StringBuffer sb = new StringBuffer();
                // 编辑结构化数据一次调用
                for (com.diagbot.biz.push.entity.Pacs pacs : inputInfo.getPacs()) {
                    sb.append(StringUtils.isNotBlank(pacs.getUniqueName())? pacs.getUniqueName() + ":":pacs.getName()+ ":").append(pacs.getResult() + "。");
                }
                // List<PacsNew> collect = inputInfo.getPacs().stream().map(x -> {
                //     PacsNew pacsNew = new PacsNew();
                //     pacsNew.setName(x.getName());
                //     pacsNew.setUniqueName(x.getUniqueName());
                //     pacsNew.setResult(x.getResult());
                //     return pacsNew;
                // }).collect(Collectors.toList());
                // wordCrfDTO.setPacsLabel(pacsLabel);
                // pacsLabel.setPacsNewList(collect);
                // for (PacsNew pacsNew : collect) {
                //     sb.append(StringUtils.isNotBlank(pacsNew.getUniqueName())? pacsNew.getUniqueName() + ":":pacsNew.getName()+ ":").append(pacsNew.getResult() + "。");
                // }
                pacsLabel.setText(sb.toString());
                pacsLabel.setAiText(sb.toString());
                putContent(crfContent, medicalTextType.get(5), sb.toString(), Content.pacs);
                wordCrfDTO.setPacsLabel(pacsLabel);
            }

            /* *//* 辅助检查(暂用主诉现病史模型,之后会新训练单独模型再做修改) *//*
            if (StringUtils.isNotBlank(inputInfo.getPacsString())) {
                String pacsText = inputInfo.getPacsString();
                PacsLabel pacsLabel = new PacsLabel();
                pacsLabel.setText(pacsText);
                pacsLabel.setAiText(pacsText);
                wordCrfDTO.setPacsLabel(pacsLabel);
                putContent(crfContent, medicalTextType.get(3), pacsText, Content.pacs);
            }
            //* 专科检查(专科体格检查)
            if (beHospitalizedDoc.getVitalLabelSpecial() != null && beHospitalizedDoc.getVitalLabelSpecial().isCrfLabel()) {
                String vitalSpecialText = beHospitalizedDoc.getVitalLabelSpecial().getText();
                putContent(crfContent, medicalTextType.get(3), vitalSpecialText, Content.special_exam);
            }*/
            /* 一般体格检查(存放一般查体) */
            if (StringUtils.isNotBlank(inputInfo.getVital())) {
                String vitalText = inputInfo.getVital();
                VitalLabel vitalLabel = new VitalLabel();
                vitalLabel.setAiText(vitalText);
                vitalLabel.setText(vitalText);
                wordCrfDTO.setVitalLabel(vitalLabel);
                putContent(crfContent, medicalTextType.get(7), vitalText, Content.phys_exam);
            }
            // 初步诊断
            if (StringUtils.isNotBlank(inputInfo.getDiagString())) {
                String initialDiagText = inputInfo.getDiagString();
                DiagLabel diagLabel = new DiagLabel();
                diagLabel.setText(initialDiagText);
                diagLabel.setAiText(initialDiagText);
                wordCrfDTO.setDiagLabel(diagLabel);
                if (StringUtil.isNotBlank(initialDiagText)) {
                    putContent(crfContent, medicalTextType.get(6), initialDiagText, Content.initial_diag);
                }
            }
            /*
            *//* 修正诊断 *//*
            if (beHospitalizedDoc.getRevisedDiagLabel() != null && beHospitalizedDoc.getRevisedDiagLabel().isCrfLabel()) {
                String revisedDiagText = beHospitalizedDoc.getRevisedDiagLabel().getText();
                *//*if (CatalogueUtil.numberExist(initial_diag_text)) {
                    revised_diag_text = CatalogueUtil.removeBetweenWordSpace(revised_diag_text);
                }*//*
                putContent(crfContent, medicalTextType.get(6), revisedDiagText, Content.revised_diag);
            }
            *//* 补充诊断 *//*
            if (beHospitalizedDoc.getSuppleDiagLabel() != null && beHospitalizedDoc.getSuppleDiagLabel().isCrfLabel()) {
                String suppleDiagText = beHospitalizedDoc.getSuppleDiagLabel().getText();
                *//*if (CatalogueUtil.numberExist(initial_diag_text)) {
                    supple_diag_text = CatalogueUtil.removeBetweenWordSpace(supple_diag_text);
                }*//*
                putContent(crfContent, medicalTextType.get(6), suppleDiagText, Content.supple_diag);
            }*/
            // CRF处理
            JSONObject midData = null;
            try {
                 midData = loadAI(crfContent, crfServiceClient);
            } catch (Exception e) {
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "CRF模型【服务器】挂了!" + e.getMessage());
            }
            /* 处理主诉 */
            if (StringUtil.isNotBlank(wordCrfDTO.getChiefLabel().getText())) {
                putChiefCrfData(midData.getJSONObject(Content.chief), wordCrfDTO);
            }
            /* 处理现病史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getPresentLabel().getText()) ) {
                putPresentCrfData(midData.getJSONObject(Content.present), wordCrfDTO);
            }
            /* 处理既往史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getPastLabel().getText())) {
                putPastCrfData(midData.getJSONObject(Content.past), wordCrfDTO);
            }
            /* 处理个人史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getPersonalLabel().getText())) {
                putPersonalCrfData(midData.getJSONObject(Content.personal), wordCrfDTO);
            }
            /* 处理月经史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getMenstrualLabel().getText())) {
                putMensesCrfData(midData.getJSONObject(Content.menses), wordCrfDTO);
            }
            /* 处理家族史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getFamilyLabel().getText())) {
                putFamilyCrfData(midData.getJSONObject(Content.family), wordCrfDTO);
            }
            /* 处理婚育史 */
            if (StringUtil.isNotBlank(wordCrfDTO.getMaritalLabel().getText())) {
                putMaritalCrfData(midData.getJSONObject(Content.marriage), wordCrfDTO);
            }
             //处理初步诊断
            if (StringUtil.isNotBlank(wordCrfDTO.getDiagLabel().getText())) {
                putInitialDiagCrfData(midData.getJSONObject(Content.initial_diag), wordCrfDTO);
            }
            //处理查体数据
            if (StringUtil.isNotBlank(wordCrfDTO.getVitalLabel().getText())) {
                putInitialVitalCrfData(midData.getJSONObject(Content.phys_exam), wordCrfDTO);
            }
             /*//处理修正诊断
            if (beHospitalizedDoc.getRevisedDiagLabel() != null && beHospitalizedDoc.getRevisedDiagLabel().isCrfLabel()) {
                putRevisedDiagCrfData(midData.getJSONObject(Content.revised_diag), inputInfo);
            }
             //处理补充诊断
            if (beHospitalizedDoc.getSuppleDiagLabel() != null && beHospitalizedDoc.getSuppleDiagLabel().isCrfLabel()) {
                putSuppleDiagCrfData(midData.getJSONObject(Content.supple_diag), inputInfo);
            }*/
            /* 处理辅助检查 */
            /*if (wordCrfDTO.getPacsLabel() != null) {
                putPacsCrfData(midData.getJSONObject(Content.pacs), wordCrfDTO);
            }*/
            // 处理辅检
            if(StringUtil.isNotBlank(wordCrfDTO.getPacsLabel().getText())) {
                putInitialPacsCrfData(midData.getJSONObject(Content.pacs), wordCrfDTO.getPacsLabel());
            }
        }
    }

    public void putChiefCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //使用现病史结构
        EntityProcessClinic entityProcessClinic = new EntityProcessClinic();
        PresentLabel presentLabel = entityProcessClinic.extractEntity(aiOut);


        ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
        chiefLabel.setClinicals(presentLabel.getClinicals());
        chiefLabel.setDiags(presentLabel.getDiags());
        chiefLabel.setPds(presentLabel.getPds());
    }

    public void putPresentCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }


        //放置入inputinfo
        EntityProcessClinic entityProcessClinic = new EntityProcessClinic();
        PresentLabel presentLabel = entityProcessClinic.extractEntity(aiOut);

        presentLabel.setText(wordCrfDTO.getPresentLabel().getText());
        List<Lis> lis = loadLises(aiOut);
        presentLabel.setLises(lis);

        EntityProcessPacs entityProcessPacs = new EntityProcessPacs();
        PacsLabel pacsLabel = presentLabel.getPacsLabel();
        if(pacsLabel == null){
            pacsLabel = new PacsLabel();
            presentLabel.setPacsLabel(pacsLabel);
        }
        entityProcessPacs.extractEntity(aiOut,pacsLabel);
        //服用药品
        List<Medicine> medicines = takeMedicine(presentLabel);
        presentLabel.setTakeMedicine(medicines);
        wordCrfDTO.setPresentLabel(presentLabel);

    }

    /**
     * 家族史
     *
     * @param jsonObject
     * @param wordCrfDTO
     */
    public void putFamilyCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //家族史信息处理
        EntityProcessFamily entityProcess = new EntityProcessFamily();
        FamilyLabel familyLabel = entityProcess.extractEntity(aiOut);
        familyLabel.setText(wordCrfDTO.getFamilyLabel().getText());
        wordCrfDTO.setFamilyLabel(familyLabel);
    }

    /**
     * 婚育史
     *
     * @param jsonObject
     * @param wordCrfDTO
     */
    public void putMaritalCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //家族史信息处理
        EntityProcessMarital entityProcess = new EntityProcessMarital();
        MaritalLabel maritalLabel = entityProcess.extractEntity(aiOut);
        maritalLabel.setText(wordCrfDTO.getMaritalLabel().getText());
        wordCrfDTO.setMaritalLabel(maritalLabel);
    }

    public void putPastCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //放置入inputinfo
        EntityProcessPast entityProcessPast = new EntityProcessPast();
        PastLabel pastLabel = entityProcessPast.extractEntity(aiOut);
        pastLabel.setText(wordCrfDTO.getPastLabel().getText());
        wordCrfDTO.setPastLabel(pastLabel);

    }

    /**
     * 个人史信息提取
     *
     * @param jsonObject
     * @param wordCrfDTO
     */
    public void putPersonalCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //个人史信息提取
        EntityProcessPersonal entityProcessPersonal = new EntityProcessPersonal();
        PersonalLabel personalLabel = entityProcessPersonal.extractEntity(aiOut);
        personalLabel.setText(wordCrfDTO.getPersonalLabel().getText());
        wordCrfDTO.setPersonalLabel(personalLabel);
    }

    /**
     * 月经史信息提取
     *
     * @param jsonObject
     * @param wordCrfDTO
     */
    public void putMensesCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //月经史信息处理
        EntityProcessMenses entityProcess = new EntityProcessMenses();
        MenstrualLabel menstrualLabel = entityProcess.extractEntity(aiOut);
        menstrualLabel.setText(wordCrfDTO.getMenstrualLabel().getText());
        wordCrfDTO.setMenstrualLabel(menstrualLabel);
    }

    /*
     * 初步诊断信息提取
     *
     * @param jsonObject
     * @param inputInfo
     */
    public void putInitialDiagCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //诊断信息
        EntityProcessDiag entityProcessDiag = new EntityProcessDiag();
        List<Diag> diags = entityProcessDiag.extractEntity(aiOut);

        DiagLabel initialDiagLabel = new DiagLabel();
        initialDiagLabel.setText(wordCrfDTO.getDiagLabel().getText());
        initialDiagLabel.setDiags(diags);
        wordCrfDTO.setDiagLabel(initialDiagLabel);
    }

    /**
     * 修正诊断信息提取
     *
     * @param jsonObject
     * @param
     *//*
    public void putRevisedDiagCrfData(JSONObject jsonObject, InputInfo inputInfo) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //诊断信息
        EntityProcessDiag entityProcessDiag = new EntityProcessDiag();
        List<Diag> diags = entityProcessDiag.extractEntity(aiOut);

        DiagLabel revisedDiagLabel = new DiagLabel();
        revisedDiagLabel.setText(inputInfo.getBeHospitalizedDoc().getRevisedDiagLabel().getText());
        revisedDiagLabel.setDiags(diags);
        inputInfo.getBeHospitalizedDoc().setRevisedDiagLabel(revisedDiagLabel);

        //替换初步诊断中的第一个诊断信息
       *//* DiagLabel initialDiagLabel = inputInfo.getBeHospitalizedDoc().getInitialDiagLabel();
        if (initialDiagLabel.getDiags().size() > 0 && diags.size() > 0) {
            initialDiagLabel.getDiags().remove(0);
            initialDiagLabel.getDiags().add(0, diags.get(0));
        }*//*
    }

    /**
     * 补充诊断信息提取
     *
     * @param jsonObject
     * @param inputInfo
     *//*
    public void putSuppleDiagCrfData(JSONObject jsonObject, InputInfo inputInfo) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //诊断信息
        EntityProcessDiag entityProcessDiag = new EntityProcessDiag();
        List<Diag> diags = entityProcessDiag.extractEntity(aiOut);

        DiagLabel suppleDiagLabel = new DiagLabel();
        suppleDiagLabel.setText(inputInfo.getBeHospitalizedDoc().getSuppleDiagLabel().getText());
        suppleDiagLabel.setDiags(diags);
        inputInfo.getBeHospitalizedDoc().setSuppleDiagLabel(suppleDiagLabel);
    }

    /**
     * 辅助检查信息抽取
     *
     * @param jsonObject
     * @param wordBillCrfDTO
     */
    /*public void putPacsCrfData(JSONObject jsonObject, WordCrfDTO wordBillCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        //放置入inputinfo
        PacsLabel pacsLabel = wordBillCrfDTO.getPacsLabel();
        pacsLabel.setPacses(loadPacses(aiOut));
    }*/

    /**
     * 关系抽取疾病信息
     *
     * @param aiOut
     * @return
     */
    public List<Diag> loadDiags(JSONObject aiOut) {
        //诊断信息
        EntityProcessDiag entityProcessDiag = new EntityProcessDiag();
        List<Diag> diags = entityProcessDiag.extractEntity(aiOut);
        return diags;
    }

    /**
     * 关系抽取化验信息
     *
     * @param aiOut
     * @return
     */
    public List<Lis> loadLises(JSONObject aiOut) {
        //化验信息
        EntityProcessLis entityProcessLis = new EntityProcessLis();
        List<Lis> lises = entityProcessLis.extractEntity(aiOut);
        return lises;
    }

    /**
     * 关系抽取辅检信息
     *
     * @param aiOut
     * @return
     */
    // public List<Pacs> loadPacses(JSONObject aiOut) {
    //     //辅检信息
    //     EntityProcessPacs entityProcessLis = new EntityProcessPacs();
    //     List<Pacs> pacses = entityProcessLis.extractEntity(aiOut);
    //     return pacses;
    // }

    /**
     * 关系抽取体格检查
     *
     * @param aiOut
     * @return
     */
    public List<Vital> loadvital(JSONObject aiOut) {
        //一般情况描述信息
        EntityProcessVital entityProcessVital = new EntityProcessVital();
        VitalLabel vitalLabel = entityProcessVital.extractEntity(aiOut);
        List<Vital> vitals = vitalLabel.getVitals();
        return vitals;
    }

    /**
     * 从个人史取最后30个字，拼接到月经史
     *
     * @param concatMenstrual
     * @param personalText
     * @return
     */
    private String getConcatMenstrual(String concatMenstrual, String personalText) {
        if (personalText != null) {
            if (personalText.length() > 30) {
                concatMenstrual = personalText.substring(personalText.length() - 30);
            } else {
                concatMenstrual = personalText;
            }
        }
        return concatMenstrual;
    }

    /*
     * 初步诊断信息提取
     *
     * @param jsonObject
     * @param inputInfo
     */
    public void putInitialVitalCrfData(JSONObject jsonObject, WordCrfDTO wordCrfDTO) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }

        //一般情况描述信息
        EntityProcessVital entityProcessVital = new EntityProcessVital();
        VitalLabel vitalLabel = entityProcessVital.extractEntity(aiOut);
        List<Vital> vitals = vitalLabel.getVitals();

        // 生命体征特殊处理
        for (Vital vi : vitals) {
            if ("T".equals(vi.getName().toUpperCase())) {
                vi.setStandName("体温");
            } else if ("P".equals(vi.getName().toUpperCase())) {
                vi.setStandName("脉搏");
            } else if ("R".equals(vi.getName().toUpperCase())) {
                vi.setStandName("呼吸");
            } else if ("血压".equals(vi.getName()) || "BP".equals(vi.getName().toUpperCase())) {
                String bpName = vi.getPd().getName();
                Pattern pattern = Pattern.compile("\\d+/\\d+"); // 示例：血压80/44mmHg
                Matcher matcher = pattern.matcher(bpName);

                if (matcher.find()) {
                    String val = matcher.group();
                    vi.getPd().setValue(val);  // 重新赋值血压的数值
                    vi.getPd().setUnit(bpName.substring(bpName.indexOf(val) + 1));   // 重新赋值血压的单位

                    String[] bp = val.split("/");

                    List<Usual> usualList = new ArrayList<>();
                    Usual usualSbp = new Usual();
                    usualSbp.setName(vi.getName());
                    usualSbp.setStandName("收缩压");
                    usualSbp.setValue(bp[0]);
                    usualList.add(usualSbp);
                    vi.setUsualList(usualList);

                    Usual usualDbp = new Usual();
                    usualDbp.setName(vi.getName());
                    usualDbp.setStandName("舒张压");
                    usualDbp.setValue(bp[1]);
                    usualList.add(usualDbp);

                    // PD sbp = new PD(); // 收缩压
                    // sbp.setValue(bp[0]);
                    // sbp.setName("收缩压");
                    // sbp.setStandName("收缩压");
                    // vitalLabel.setSbp(sbp);

                    // PD dbp = new PD(); // 舒张压
                    // dbp.setValue(bp[1]);
                    // dbp.setName("舒张压");
                    // dbp.setStandName("舒张压");
                    // vitalLabel.setDbp(dbp);
                }
            }
        }
        wordCrfDTO.setVitalLabel(vitalLabel);
    }

    /*
     * 辅检信息提取
     *
     * @param jsonObject
     * @param inputInfo
     */
    public void putInitialPacsCrfData(JSONObject jsonObject, PacsLabel pacsLabel) {
        if (jsonObject == null) {
            return;
        }
        JSONObject aiOut = jsonObject.getJSONObject(entityRelationObject).getJSONObject(BeHospitalizedAI.outputs);
        if (aiOut == null) {
            return;
        }
        EntityProcessPacs entityProcessPacs = new EntityProcessPacs();
        entityProcessPacs.extractEntity(aiOut,pacsLabel);
//        EntityProcessPacsNew entityProcessPacsNew = new EntityProcessPacsNew();
        // Map<String, PacsNew> map = entityProcessPacsNew.extractEntity(aiOut);
        //
        // List<PacsNew> pacsNewList = pacsLabel.getPacsNewList();
        // for (PacsNew p : pacsNewList) {
        //     if (map.get(p.getName()) != null) {
        //         p.setPacsResults(map.get(p.getName()).getPacsResults());
        //         p.setDisease(map.get(p.getName()).getDisease());
        //     }
        // }
    }

    /**
     * 一般情况以后的药品
     * @param presentLabel
     */
    public List<Medicine> takeMedicine(PresentLabel presentLabel){
        List<Medicine> takems = new ArrayList<>();
        List<GeneralDesc> generals = presentLabel.getGenerals();
        List<Medicine> medicines = presentLabel.getMedicines();
        if (generals.size() > 0) {
            String presentText = presentLabel.getText();
            if (StringUtils.isNotBlank(presentText) && medicines != null && medicines.size() > 0) {
                String lastGeneral = generals.get(generals.size() - 1).getName();
                int lastGeneralIndex = presentText.lastIndexOf(lastGeneral);
                for (Medicine medicine : medicines) {
                    /* 现病史中一般情况之后的药品名称,并且不包含不详 */
                    int medicine_index = presentText.indexOf(medicine.getName());
                    if (medicine_index > lastGeneralIndex && !medicine.getName().contains("不详")) {
                        takems.add(medicine);
                    }
                }
            }
        }
        return takems;
    }
}